/*
    ext2resize.c -- ext2 resizer frontend
    Copyright (C) 1998, 1999, 2000 Lennert Buytenhek <buytenh@gnu.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static const char _ext2resize_c[] = "$Id: ext2resize.c,v 1.10 2000/03/22 21:51:02 adilger Exp $";

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "ext2.h"

struct ext2_fs		*fs;
int			 opt_debug = 0;
int			 opt_force = 0;
int			 opt_ignore = 0;
int			 opt_safe = 1;
int			 opt_verbose = 0;
char			*progname;

static void print_version(FILE *outfile)
{
	fprintf(outfile, "ext2resize v%s - 1999/08/28 for EXT2FS %s\n",
		VERSION, EXT2FS_VERSION);
}

void usage(FILE *outfile)
{
	fprintf(outfile,
		"usage: %s [-d] [-f] [-u] [-v] device new_size[bkmgt]\n\t"
		"\t-d, --debug    : turn debug info on\n"
		"\t-f, --force    : skip checks\n"
		"\t-i, --ignore   : ignore device size returned by ioctl\n"
		"\t-u, --unsafe   : skip buffer cache flushing\n"
		"\t-v, --verbose  : be verbose\n"
		"\tnew_size is in ext2 filesystem blocks (1k, 2k, or 4k),\n"
		"\tdisk Blocks (512 byte), Kilo-, Mega-, Giga-, or Terabytes\n",
		progname);
	exit(1);
}

void parse_args(int *argc, char **argv[])
{
	static struct option long_options[] = {
		{"debug", 0, 0, 'd'},
		{"force", 0, 0, 'f'},
		{"ignore", 0, 0, 'i'},
		{"unsafe", 0, 0, 'u'},
		{"verbose", 0, 0, 'v'},
		{NULL, 0, 0, 0}};
	int c;
	int dummy;

	opt_force = 0;

	while ((c = getopt_long(*argc, *argv, "dfiuv", long_options, &dummy)) != -1)
	{
		switch (c)
		{
		case 'd': opt_debug = 1; break;
		case 'f': opt_force = 1; break;
		case 'i': opt_ignore = 1; break;
		case 'u': opt_safe = 0; break;
		case 'v': opt_verbose = 1; break;
		default:  fprintf(stderr, "%s: unknown option %c\n",
				  progname, c);
			  usage(stderr);
		}
	}

	*argc -= optind;
	*argv += optind;
}

int main(int argc, char *argv[])
{
	struct ext2_dev_handle	*handle;
	int			 newgroups;
	char			*dev;
	char			 mod = '\0';
	blk_t			 newsize;
	blk_t			 devsize;

	progname = strrchr(argv[0], '/') ? strrchr(argv[0], '/') + 1 : argv[0];
	print_version(stdout);
	parse_args(&argc, &argv);

	if (argc < 1 || argc > 2)
		usage(stderr);

	dev = argv[0];

	handle = ext2_make_dev_handle_from_file(dev);
	if (handle == NULL)
	{
		fprintf(stderr, "can't open %s\n", dev);
		return 1;
	}

	if (argc == 2)
	{
		sscanf(argv[1], "%i%c", &newsize, &mod);

		switch (mod)	/* Order of these options is important!! */
		{
			case 't':
			case 'T': newsize *= 1024; /* no break */
			case 'g':
			case 'G': newsize *= 1024; /* no break */
			case 'm':
			case 'M': newsize *= 1024; /* no break */
			case 'k': /* Need to open fs to find block size */
			case 'K':
			case  0 : break;
			case 'b':
			case 'B': newsize /= 2; break;
			default: usage(stderr);
		}

		newgroups = howmany(newsize, 8192); /* worst case 1k blocks */
		if (ext2_max_groups < newgroups)
			ext2_max_groups = newgroups;
	}

	if ((fs = ext2_open(handle,
			    opt_force ? (EXT2_ERROR_FS | EXT2_VALID_FS) : 0))
	    == NULL)
	{
		fprintf(stderr, "can't open %s\n", argv[0]);
		exit(-1);
	}

	if (mod != '\0')
		newsize /= fs->blocksize / 1024;

	devsize = fs->devhandle->ops->get_size(fs->devhandle->cookie);

	if (argc == 1)	/* Resize to the full size if none is given */
		newsize = devsize;
	else if (newsize > devsize && !opt_ignore)
	{
		/* We could put a system() call here to lvextend with the
		 * appropriate parameters to grow the LV before the resize.
		 */
		fprintf(stderr, "%s: %s has %u blocks not %u, resize aborted\n",
			progname, dev, devsize, newsize);
		return 1;
	}

	fs->opt_debug = opt_debug;
	fs->opt_safe = opt_safe;
	fs->opt_verbose = opt_verbose;

	ext2_resize_fs(fs, newsize);
	ext2_close(fs);

	return 0;
}
